/*************************************************************************************************
 * Common modules related to estwaver
 *                                                      Copyright (C) 2004-2006 Mikio Hirabayashi
 * This file is part of Hyper Estraier.
 * Hyper Estraier is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License or any later version.  Hyper Estraier is distributed in the hope
 * that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with Hyper
 * Estraier; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA.
 *************************************************************************************************/


#ifndef _WAVERMOD_H                      /* duplication check */
#define _WAVERMOD_H

#include "estraier.h"
#include "estmtdb.h"
#include "estnode.h"
#include "myconf.h"
#include "mymorph.h"



/*************************************************************************************************
 * pseudo API
 *************************************************************************************************/


#define NUMBUFSIZ      32                /* size of a buffer for a number */
#define URIBUFSIZ      8192              /* size of a buffer for an URI */
#define MINIBNUM       31                /* bucket number of a small map */
#define RESLIMSIZE     (1048576*32)      /* limitation of the entity body of response */

typedef struct {                         /* type of structure for priority queue */
  VILLA *db;                             /* internal database */
  double max;                            /* maximum point */
} QUEUE;

typedef struct {                         /* type of structure for a keyword */
  const char *word;                      /* face of keyword */
  int wsiz;                              /* size of the keyword */
  int pt;                                /* score tuned by TF-IDF */
} KEYSC;

enum {                                   /* enumeration for file formats */
  FF_TEXT,                               /* plain text */
  FF_HTML,                               /* HTML */
  FF_MIME,                               /* MIME */
  FF_DRAFT,                              /* draft */
  FF_UNKNOWN                             /* unknown */
};


/* Open a priority queue.
   `name' specifies the name of a database file.
   The return value is the queue handle or `NULL' on failure. */
QUEUE *queue_open(const char *name);


/* Close a priority queue.
   `queue' specifies a queue handle.
   The return value is true if successful, or false on failure. */
int queue_close(QUEUE *queue);


/* Set the range of the priority space of a priority queue.
   `queue' specifies a queue handle.
   `range' specifies the range of the priority space. */
void queue_set_range(QUEUE *queue, double range);


/* Enqueue a record into a priority queue.
   `queue' specifies a queue handle.
   `str' specifies a string.
   `priority' specifies the priority between 0.0 and 1.0.
   The return value is true if successful, or false on failure. */
int queue_enqueue(QUEUE *queue, const char *str, double priority);


/* Dequeue a record from a priority queue.
   `queue' specifies a queue handle.
   The return value is the pointer to a record or `NULL' if no record exists.
   Because the region of the return value is allocated with the `malloc' call, it should be
   released with the `free' call if it is no longer in use. */
char *queue_dequeue(QUEUE *queue);


/* Add a word to a keyword map.
   `kwmap' specifies a keyword map handle.
   `word' specifies the string of a word.
   `frequency' specifies the frequency of the word. */
void kwmap_add(CBMAP *kwmap, const char *word, int frequency);


/* Reduce elements of a keyword map.
   `kwmap' specifies a keyword map handle.
   `num' specifies the number of elements after reduction of the keyword map. */
void kwmap_reduce(CBMAP *kwmap, int num);


/* Fetch a document of a URL.
   `url' specifies the URL of a document.
   `pxhost' specifies the host name of a proxy.  If it is `NULL', it is not used.
   `pxport' specifies the port number of the proxy.
   `outsec' specifies timeout in seconds.  If it is negative, it is not used.
   `raw' specifies a datum handle to store raw data.  If it is `NULL', it is not used.
   `heads' specifies a map handle to store HTTP headers.  If it is `NULL', it is not used.
   `links' specifies a list handle to store links.  If it is `NULL', it is not used.
   `doc' specifies a document handle to store attributes and texts.  If it is `NULL', it is not
   used.
   The return value is true if success, else it is false. */
int fetch_document(const char *url, const char *pxhost, int pxport, int outsec,
                   CBDATUM *raw, CBMAP *heads, CBLIST *links, ESTDOC *doc, int lang);



#endif                                   /* duplication check */


/* END OF FILE */
